(* file: re.lex *)
(* description: lexer generator for regular expression *)
(* author: Takahito Aoto *)

type pos = int;
type svalue = Tokens.svalue
type ('a,'b) token = ('a,'b) Tokens.token
type lexresult  = (svalue,pos) token

val text = ref (nil: string list)
val addText = fn s => (text := s::(!text))
val lin = ref 1;
val col = ref 0; 
fun error (msg,line,col) = print ("[line "^Int.toString line^", char "^Int.toString col^"] "^msg^"\n");
val eof = fn _ => (if not (null (!text)) then error ("the file ends without closing dquote",!lin,!col) else ();
		    Tokens.EOF(!lin,!col))
val eolpos = ref 0;
%%
%header (functor RegExpLexFun (structure Tokens: RegExp_TOKENS));
%s MAIN;
alphanum = [a-zA-Z0-9];
symbolorg = [!|#*$%&+/;:<=>?@^_`{}~]|\-|\.|\]|\[|\\|\";
symbol =    [!#$%&/;:=?@^_`{}~]|\-|\]|\[|\\;
dash = "'";
idchar={alphanum}|{symbol}|{dash};
eol = ("\013\010"|"\010"|"\013");
ws = [\ \t\r];
%%
<INITIAL>{ws}*        => (lin:=1; eolpos:=0; YYBEGIN MAIN; continue());
<MAIN>{ws}+           => (lex ());
<MAIN>{eol}           => (lin:=(!lin)+1; eolpos:=yypos+size yytext; lex ());
<MAIN>"("             => (col:=yypos-(!eolpos); Tokens.LPAREN(!lin,!col));
<MAIN>")"             => (col:=yypos-(!eolpos); Tokens.RPAREN(!lin,!col));
<MAIN>"*"             => (col:=yypos-(!eolpos); Tokens.AST(!lin,!col));
<MAIN>"+"             => (col:=yypos-(!eolpos); Tokens.PLUS(!lin,!col));
<MAIN>"|"             => (col:=yypos-(!eolpos); Tokens.VERT(!lin,!col));
<MAIN>"."             => (col:=yypos-(!eolpos); Tokens.DOT(!lin,!col));
<MAIN>"<"             => (col:=yypos-(!eolpos); Tokens.LANGLE(!lin,!col));
<MAIN>">"             => (col:=yypos-(!eolpos); Tokens.RANGLE(!lin,!col));
<MAIN>"\""            => (col:=yypos-(!eolpos); Tokens.DQUOTE(!lin,!col));
<MAIN>{idchar}      => (col:=yypos-(!eolpos); Tokens.ID (yytext,!lin,!col));
<MAIN>.               => (col:=yypos-(!eolpos); error ("ignoring illegal character" ^ yytext,!lin,!col); lex());
