(* file: tpdb.lex *)
(* description: lexer generator *)
(* author: Takahito Aoto *)

type pos = int;
type svalue = Tokens.svalue
type ('a,'b) token = ('a,'b) Tokens.token
type lexresult  = (svalue,pos) token

val text = ref (nil: string list)
val addText = fn s => (text := s::(!text))
val lin = ref 1;
val col = ref 0; 
fun error (msg,line,col) = print ("[line "^Int.toString line^", char "^Int.toString col^"] "^msg^"\n");
val eof = fn _ => (if not (null (!text)) then error ("the file ends without closing dquote",!lin,!col) else ();
		    Tokens.EOF(!lin,!col))
val eolpos = ref 0;
%%
%header (functor TpdbLexFun (structure Tokens: Tpdb_TOKENS));
%s MAIN STRING;
alphanum = [a-zA-Z0-9];
symbol = [!|#*$%&+/;:<=>?@^_`{}~]|\-|\.|\]|\[|\\|\";
dash = "'";
idchar={alphanum}|{symbol}|{dash};
eol = ("\013\010"|"\010"|"\013");
ws = [\ \t\r];
%%
<INITIAL>{ws}*        => (lin:=1; eolpos:=0; YYBEGIN MAIN; continue());
<MAIN>{ws}+           => (lex ());
<MAIN>{eol}           => (lin:=(!lin)+1; eolpos:=yypos+size yytext; lex ());
<MAIN>"->"            => (col:=yypos-(!eolpos); Tokens.ARROW(!lin,!col));
<MAIN>"("             => (col:=yypos-(!eolpos); Tokens.LPAREN(!lin,!col));
<MAIN>")"             => (col:=yypos-(!eolpos); Tokens.RPAREN(!lin,!col));
<MAIN>","             => (col:=yypos-(!eolpos); Tokens.COMMA(!lin,!col));
<MAIN>{idchar}+       => (col:=yypos-(!eolpos); 
                          case yytext of 
                               "STATES" => Tokens.STATES(!lin,!col)
                             | "ALPHABET" => Tokens.ALPHABET(!lin,!col) 
                             | "TRANSITIONS" => Tokens.TRANSITIONS(!lin,!col) 
                             | "START" => Tokens.START(!lin,!col) 
                             | "ACCEPT" => Tokens.ACCEPT(!lin,!col) 
                             | _ => Tokens.ID (yytext,!lin,!col)
                          );
<MAIN>.               => (col:=yypos-(!eolpos); error ("ignoring illegal character" ^ yytext,!lin,!col); lex());
